==> ievv_opensource-5.19.0.tar.gz: ievv_opensource-5.19.0/ievv_opensource/ievv_logging/utils.py <==
"""
An easy to use interface for logging events to the database models in this app.
"""
from django.utils import timezone

from .models import IevvLoggingEventBase, IevvLoggingEventItem


def getDuration(from_dt, to_dt, interval='default'):
    """
    Examples:

        from_dt = datetime(2019, 3, 5, 23, 8, 15)
        to_dt = datetime.now()

        print(getDuration(from_dt, to_dt))
        print(getDuration(from_dt, to_dt, interval='seconds'))
        print(getDuration(from_dt, to_dt, interval='minutes'))
    """
    duration = to_dt - from_dt
    duration_in_s = duration.total_seconds()

    def years():
        return divmod(duration_in_s, 31556926) # Seconds in a year=31556926.

    def days(seconds = None):
        return divmod(seconds or duration_in_s, 86400) # Seconds in a day = 86400

    def hours(seconds = None):
        return divmod(seconds or duration_in_s, 3600) # Seconds in an hour = 3600

    def minutes(seconds = None):
        return divmod(seconds or duration_in_s, 60) # Seconds in a minute = 60

    def seconds(seconds = None):
        if seconds:
            return divmod(seconds, 1)
        return duration_in_s

    def totalDuration():
        y = years()
        d = days(y[1]) # Use remainder to calculate next variable
        h = hours(d[1])
        m = minutes(h[1])
        s = seconds(m[1])
        duration_string = "{} years, {} days, {} hours, {} minutes and {} seconds".format(
            int(y[0]), int(d[0]), int(h[0]), int(m[0]), int(s[0])
        )
        if duration_string.startswith('0 years'):
            duration_string = duration_string.replace('0 years, ', '')
            if duration_string.startswith('0 days'):
                duration_string = duration_string.replace('0 days, ', '')
        return duration_string

    return {
        'minutes': int(minutes()[0]),
        'seconds': int(seconds()),
        'default': totalDuration()
    }[interval]


class IevvLogging():
    """
    By using this you can easily log and have a useful overview about what
    is going on in e.g. scheduled scripts.

    To use, add ievv_opensource.ievv_logging to INSTALLED_APPS. Run migrate, and for example
    if you are running a management script called "the_foo_script" nightly, do:

        from ievv_opensource.ievv_logging.utils import IevvLogging

        ievvlogging = IevvLogging('the_foo_script')
        ievvlogging.begin()

        # the script does its work

        ievvlogging.finish(
            number_of_users_updated=12,
            number_of_users_anonymized=4
        )
    """

    def __init__(self, slug):
        self.slug = slug

    def _get_logging_base_from_slug(self):
        if IevvLoggingEventBase.objects.filter(slug=self.slug).exists():
            return IevvLoggingEventBase.objects.get(slug=self.slug)
        return IevvLoggingEventBase(slug=self.slug)

    def begin(self):
        self.loggingbase = self._get_logging_base_from_slug()
        self.loggingbase.last_started = timezone.now()
        self.loggingbase.last_finished = None
        self.loggingbase.save()

    def finish(self, **kwargs):
        finish = timezone.now()
        start = self.loggingbase.last_started
        duration_string = getDuration(start, finish)
        duration_seconds = getDuration(start, finish, 'seconds')
        # update the base model
        self.loggingbase.last_finished = finish
        self.loggingbase.time_spent = duration_string
        self.loggingbase.time_spent_in_seconds = duration_seconds
        self.loggingbase.save()
        # create item model instance
        loggingitem = IevvLoggingEventItem(
            logging_base=self.loggingbase,
            time_spent=duration_string,
            time_spent_in_seconds=duration_seconds,
            data=kwargs,
            start_datetime=start,
            end_datetime=finish
        )
        loggingitem.save()

==> ievv_opensource-5.19.1.tar.gz: ievv_opensource-5.19.1/ievv_opensource/ievv_logging/utils.py <==
"""
An easy to use interface for logging events to the database models in this app.
"""
from django.utils import timezone

from .models import IevvLoggingEventBase, IevvLoggingEventItem


def getDuration(from_dt, to_dt, interval='default'):
    """
    Examples:

        from_dt = datetime(2019, 3, 5, 23, 8, 15)
        to_dt = datetime.now()

        print(getDuration(from_dt, to_dt))
        print(getDuration(from_dt, to_dt, interval='seconds'))
        print(getDuration(from_dt, to_dt, interval='minutes'))
    """
    duration = to_dt - from_dt
    duration_in_s = duration.total_seconds()

    def years():
        return divmod(duration_in_s, 31556926) # Seconds in a year=31556926.

    def days(seconds = None):
        return divmod(seconds or duration_in_s, 86400) # Seconds in a day = 86400

    def hours(seconds = None):
        return divmod(seconds or duration_in_s, 3600) # Seconds in an hour = 3600

    def minutes(seconds = None):
        return divmod(seconds or duration_in_s, 60) # Seconds in a minute = 60

    def seconds(seconds = None):
        if seconds:
            return divmod(seconds, 1)
        return duration_in_s

    def totalDuration():
        y = years()
        d = days(y[1]) # Use remainder to calculate next variable
        h = hours(d[1])
        m = minutes(h[1])
        s = seconds(m[1])
        duration_string = "{} years, {} days, {} hours, {} minutes and {} seconds".format(
            int(y[0]), int(d[0]), int(h[0]), int(m[0]), int(s[0])
        )
        if duration_string.startswith('0 years'):
            duration_string = duration_string.replace('0 years, ', '')
            if duration_string.startswith('0 days'):
                duration_string = duration_string.replace('0 days, ', '')
        if duration_string == '0 hours, 0 minutes and 0 seconds':
            duration_string = 'Less than a second'
        return duration_string

    return {
        'minutes': int(minutes()[0]),
        'seconds': int(seconds()),
        'default': totalDuration()
    }[interval]


class IevvLogging():
    """
    By using this you can easily log and have a useful overview about what
    is going on in e.g. scheduled scripts.

    To use, add ievv_opensource.ievv_logging to INSTALLED_APPS. Run migrate, and for example
    if you are running a management script called "the_foo_script" nightly, do:

        from ievv_opensource.ievv_logging.utils import IevvLogging

        ievvlogging = IevvLogging(__name__) # or any custom preferred name like 'the_foo_script'
        ievvlogging.begin()

        # the script does its work

        ievvlogging.finish(
            number_of_users_updated=12,
            number_of_users_anonymized=4
        )
    """

    def __init__(self, slug):
        self.slug = slug

    def _get_logging_base_from_slug(self):
        if IevvLoggingEventBase.objects.filter(slug=self.slug).exists():
            return IevvLoggingEventBase.objects.get(slug=self.slug)
        return IevvLoggingEventBase(slug=self.slug)

    def begin(self):
        self.loggingbase = self._get_logging_base_from_slug()
        self.loggingbase.last_started = timezone.now()
        self.loggingbase.last_finished = None
        self.loggingbase.save()

    def finish(self, **kwargs):
        finish = timezone.now()
        start = self.loggingbase.last_started
        duration_string = getDuration(start, finish)
        duration_seconds = getDuration(start, finish, 'seconds')
        # update the base model
        self.loggingbase.last_finished = finish
        self.loggingbase.time_spent = duration_string
        self.loggingbase.time_spent_in_seconds = duration_seconds
        self.loggingbase.save()
        # create item model instance
        loggingitem = IevvLoggingEventItem(
            logging_base=self.loggingbase,
            time_spent=duration_string,
            time_spent_in_seconds=duration_seconds,
            data=kwargs,
            start_datetime=start,
            end_datetime=finish
        )
        loggingitem.save()
